@@ -1,5 +1,5 @@
 /*
- * Copyright (c) 2001, 2020, Oracle and/or its affiliates. All rights reserved.
+ * Copyright (c) 2001, 2023, Oracle and/or its affiliates. All rights reserved.
  * DO NOT ALTER OR REMOVE COPYRIGHT NOTICES OR THIS FILE HEADER.
  *
  * This code is free software; you can redistribute it and/or modify it
@@ -28,6 +28,10 @@ package com.sun.javatest.regtest.config;
 import java.io.File;
 import java.io.IOException;
 import java.net.MalformedURLException;
+import java.net.URL;
+import java.nio.file.Files;
+import java.nio.file.InvalidPathException;
+import java.nio.file.Path;
 import java.util.ArrayList;
 import java.util.Arrays;
 import java.util.Collection;
@@ -37,12 +41,14 @@ import java.util.HashSet;
 import java.util.LinkedHashMap;
 import java.util.List;
 import java.util.Map;
+import java.util.Objects;
 import java.util.Set;
 import java.util.function.Consumer;
 import java.util.function.Function;
 import java.util.regex.Pattern;
 import java.util.regex.PatternSyntaxException;
 import java.util.stream.Collectors;
+import java.util.stream.Stream;
 
 import com.sun.interview.Interview;
 import com.sun.interview.Question;
@@ -59,7 +65,9 @@ import com.sun.javatest.interview.BasicInterviewParameters;
 import com.sun.javatest.regtest.agent.JDK_Version;
 import com.sun.javatest.regtest.agent.SearchPath;
 import com.sun.javatest.regtest.exec.TimeoutHandlerProvider;
+import com.sun.javatest.regtest.util.FileUtils;
 import com.sun.javatest.regtest.util.StringUtils;
+import com.sun.javatest.util.I18NResourceBundle;
 
 import static com.sun.javatest.regtest.util.StringUtils.join;
 
@@ -98,7 +106,7 @@ public class RegressionParameters
     //---------------------------------------------------------------------
 
     public void setTests(Collection<String> tests) {
-        setTests(tests == null ? null : tests.toArray(new String[tests.size()]));
+        setTests(tests == null ? null : tests.toArray(new String[0]));
     }
 
     public void setTests(String[] tests) {
@@ -136,10 +144,10 @@ public class RegressionParameters
         mtfp.setTimeoutFactor(tfac);
     }
 
-    public void setExcludeLists(File[] files) {
+    public void setExcludeLists(Path[] files) {
         MutableExcludeListParameters mep =
             (MutableExcludeListParameters) getExcludeListParameters();
-        mep.setExcludeFiles(files);
+        mep.setExcludeFiles(FileUtils.toFiles(files));
     }
 
     public void setPriorStatusValues(boolean[] b) {
@@ -212,7 +220,7 @@ public class RegressionParameters
             if (mlf != null)
                 filters.add(mlf);
 
-            final TestFilter f = new CompositeFilter(filters.toArray(new TestFilter[filters.size()]));
+            final TestFilter f = new CompositeFilter(filters.toArray(new TestFilter[0]));
             return new CachingTestFilter(f.getName(), f.getDescription(), f.getReason()) {
                 @Override
                 protected String getCacheKey(TestDescription td) {
@@ -233,7 +241,7 @@ public class RegressionParameters
 
     TestFilter getModulesFilter() {
         JDK jdk = getTestJDK();
-        if (jdk == null || jdk.getVersion(this, logger).compareTo(JDK_Version.V9) == -1)
+        if (jdk == null || jdk.getVersion(this, logger).compareTo(JDK_Version.V9) < 0)
             return null;
 
         final Set<String> availModules = jdk.getSystemModules(this, logger);
@@ -373,9 +381,9 @@ public class RegressionParameters
             //    OSNAME-ARCH   Specific on to one OSNAME and ARCH, e.g. solaris-x64
             //    OSNAME-REV    Specific on to one OSNAME and REV, e.g. solaris-5.8
             Set<String> platforms = new HashSet<>();
-            for (String p: Arrays.asList(os.name, os.name.replaceAll("\\s", ""), os.family, "generic")) {
-                for (String q: Arrays.asList(null, os.arch, os.simple_arch, os.version, os.simple_version, "all")) {
-                    String ep = (q == null) ? p : p + "-" + q;
+            for (String p: List.of(os.name, os.name.replaceAll("\\s", ""), os.family, "generic")) {
+                for (String q: List.of("", os.arch, os.simple_arch, os.version, os.simple_version, "all")) {
+                    String ep = q.isEmpty() ? p : p + "-" + q;
                     platforms.add(ep.toLowerCase());
                 }
             }
@@ -462,13 +470,15 @@ public class RegressionParameters
 
     private TestFilter getMatchListFilter() {
         if (matchListFilter == UNSET) {
-            List<File> matchList = getMatchLists();
+            List<Path> matchList = getMatchLists();
             if (matchList.isEmpty()) {
                 matchListFilter = null;
             } else {
                 final ExcludeList el;
                 try {
-                    el = new ExcludeList(matchList.toArray(new File[]{}));
+                    el = new ExcludeList(matchList.stream()
+                            .map(Path::toFile)
+                            .toArray(File[]::new));
                 } catch (ExcludeList.Fault | IOException e) {
                     throw new Error(e);
                 }
@@ -560,7 +570,7 @@ public class RegressionParameters
         }
     };
 
-    private static KeywordsTestFilter UNSET_KEYWORDS_FILTER = new KeywordsTestFilter(UNSET);
+    private static final KeywordsTestFilter UNSET_KEYWORDS_FILTER = new KeywordsTestFilter(UNSET);
 
     @Override
     public TestFilter getPriorStatusFilter() {
@@ -619,106 +629,122 @@ public class RegressionParameters
     private static final String TIMEOUT_HANDLER = ".timeoutHandler";
     private static final String TIMEOUT_HANDLER_PATH = ".timeoutHandlerPath";
     private static final String TIMEOUT_HANDLER_TIMEOUT = ".timeoutHandlerTimeout";
+    private static final String CUSTOM_TEST_THREAD_FACTORY = ".testThreadFactory";
+    private static final String CUSTOM_TEST_THREAD_FACTORY_PATH = ".testThreadFactoryPath";
+    private static final String TEST_QUERIES = ".testQueries";
 
-    @Override @SuppressWarnings({"rawtypes", "unchecked"})
-    public void load(Map data, boolean checkChecksum) throws Interview.Fault {
+    @Override
+    public void load(Map<String, String> data, boolean checkChecksum) throws Interview.Fault {
         super.load(data, checkChecksum);
-        load0((Map<String,String>) data, checkChecksum);
-    }
-
-    private void load0(Map<String,String> data, boolean checkChecksum) throws Interview.Fault {
         String prefix = getTag();
 
-        String v;
+        try {
+            String v;
 
-        v = data.get(prefix + ENVVARS);
-        if (v != null)
-            setEnvVars(deserializeEnv(v, "\n"));
+            v = data.get(prefix + ENVVARS);
+            if (v != null)
+                setEnvVars(deserializeEnv(v, "\n"));
 
-        v = data.get(prefix + CHECK);
-        if (v != null)
-            setCheck(v.equals("true"));
+            v = data.get(prefix + CHECK);
+            if (v != null)
+                setCheck(v.equals("true"));
 
-        v = data.get(prefix + EXEC_MODE);
-        if (v != null)
-            setExecMode(ExecMode.valueOf(v));
+            v = data.get(prefix + EXEC_MODE);
+            if (v != null)
+                setExecMode(ExecMode.valueOf(v));
 
-        v = data.get(prefix + IGNORE);
-        if (v != null)
-            setIgnoreKind(IgnoreKind.valueOf(v));
+            v = data.get(prefix + IGNORE);
+            if (v != null)
+                setIgnoreKind(IgnoreKind.valueOf(v));
 
-        v = data.get(prefix + COMPILE_JDK);
-        if (v != null)
-            setCompileJDK(JDK.of(v));
+            v = data.get(prefix + COMPILE_JDK);
+            if (v != null)
+                setCompileJDK(JDK.of(v));
 
-        v = data.get(prefix + TEST_JDK);
-        if (v != null)
-            setTestJDK(JDK.of(v));
+            v = data.get(prefix + TEST_JDK);
+            if (v != null)
+                setTestJDK(JDK.of(v));
 
-        v = data.get(prefix + TEST_VM_OPTIONS);
-        if (v != null && v.length() > 0)
-            setTestVMOptions(Arrays.asList(StringUtils.splitSeparator("\n", v)));
+            v = data.get(prefix + TEST_VM_OPTIONS);
+            if (v != null && v.length() > 0)
+                setTestVMOptions(List.of(StringUtils.splitSeparator("\n", v)));
 
-        v = data.get(prefix + TEST_COMPILER_OPTIONS);
-        if (v != null && v.length() > 0)
-            setTestCompilerOptions(Arrays.asList(StringUtils.splitSeparator("\n", v)));
+            v = data.get(prefix + TEST_COMPILER_OPTIONS);
+            if (v != null && v.length() > 0)
+                setTestCompilerOptions(List.of(StringUtils.splitSeparator("\n", v)));
 
-        v = data.get(prefix + TEST_JAVA_OPTIONS);
-        if (v != null && v.length() > 0)
-            setTestJavaOptions(Arrays.asList(StringUtils.splitSeparator("\n", v)));
+            v = data.get(prefix + TEST_JAVA_OPTIONS);
+            if (v != null && v.length() > 0)
+                setTestJavaOptions(List.of(StringUtils.splitSeparator("\n", v)));
 
-        v = data.get(prefix + RETAIN_ARGS);
-        if (v != null && v.length() > 0)
-            setRetainArgs(Arrays.asList(StringUtils.splitSeparator("\n", v)));
+            v = data.get(prefix + RETAIN_ARGS);
+            if (v != null && v.length() > 0)
+                setRetainArgs(List.of(StringUtils.splitSeparator("\n", v)));
 
-        v = data.get(prefix + JUNIT);
-        if (v != null)
-            setJUnitPath(new SearchPath(v));
+            v = data.get(prefix + JUNIT);
+            if (v != null)
+                setJUnitPath(new SearchPath(v));
 
-        v = data.get(prefix + TESTNG);
-        if (v != null)
-            setTestNGPath(new SearchPath(v));
+            v = data.get(prefix + TESTNG);
+            if (v != null)
+                setTestNGPath(new SearchPath(v));
 
-        v = data.get(prefix + ASMTOOLS);
-        if (v != null)
-            setAsmToolsPath(new SearchPath(v));
+            v = data.get(prefix + ASMTOOLS);
+            if (v != null)
+                setAsmToolsPath(new SearchPath(v));
 
-        v = data.get(prefix + TIMELIMIT);
-        if (v != null)
-            setTimeLimit(Integer.parseInt(v));
+            v = data.get(prefix + TIMELIMIT);
+            if (v != null)
+                setTimeLimit(Integer.parseInt(v));
 
-        v = data.get(prefix + REPORTDIR);
-        if (v != null)
-            setReportDir(new File(v));
+            v = data.get(prefix + REPORTDIR);
+            if (v != null)
+                setReportDir(Path.of(v));
 
-        v = data.get(prefix + EXCLUSIVE_LOCK);
-        if (v != null)
-            setExclusiveLock(new File(v));
+            v = data.get(prefix + EXCLUSIVE_LOCK);
+            if (v != null)
+                setExclusiveLock(Path.of(v));
 
-        v = data.get(prefix + NATIVEDIR);
-        if (v != null)
-            setNativeDir(new File(v));
+            v = data.get(prefix + NATIVEDIR);
+            if (v != null)
+                setNativeDir(Path.of(v));
 
-        v = data.get(prefix + TIMEOUT_HANDLER);
-        if (v != null)
-            setTimeoutHandler(v);
+            v = data.get(prefix + TIMEOUT_HANDLER);
+            if (v != null)
+                setTimeoutHandler(v);
 
-        v = data.get(prefix + TIMEOUT_HANDLER_PATH);
-        if (v != null)
-            setTimeoutHandlerPath(v);
+            v = data.get(prefix + TIMEOUT_HANDLER_PATH);
+            if (v != null)
+                setTimeoutHandlerPath(v);
 
-        v = data.get(prefix + TIMEOUT_HANDLER_TIMEOUT);
-        if (v != null)
-            setTimeoutHandlerTimeout(v);
+            v = data.get(prefix + TIMEOUT_HANDLER_TIMEOUT);
+            if (v != null)
+                setTimeoutHandlerTimeout(v);
+
+            v = data.get(prefix + CUSTOM_TEST_THREAD_FACTORY);
+            if (v != null)
+                setTestThreadFactory(v);
+
+            v = data.get(prefix + CUSTOM_TEST_THREAD_FACTORY_PATH);
+            if (v != null)
+                setTestThreadFactoryPath(v);
+
+            v = data.get(prefix + TEST_QUERIES);
+            if (v != null) {
+                setTestQueries(List.of(StringUtils.splitSeparator("\n", v)));
+            }
+
+        } catch (InvalidPathException e) {
+            // This is unlikely to happen, but pretty serious if it does.
+            // Since we only put valid paths into the parameters, there should be
+            // no issue retrieving them after the save-load sequence.
+            throw new Interview.Fault(i18n, "rp.badPath", e.getInput(), e.getMessage());
+        }
     }
 
-    @Override @SuppressWarnings({"unchecked", "rawtypes"})
-    public void save(Map data) {
-        save0((Map<String, String>) data);
+    public void save(Map<String, String> data) {
         super.save(data);
-    }
 
-    private void save0(Map<String, String> data) {
         String prefix = getTag();
 
         if (envVars != null)
@@ -759,13 +785,13 @@ public class RegressionParameters
             data.put(prefix + TIMELIMIT, String.valueOf(timeLimit));
 
         if (reportDir != null)
-            data.put(prefix + REPORTDIR, reportDir.getPath());
+            data.put(prefix + REPORTDIR, reportDir.toString());
 
         if (exclusiveLock != null)
-            data.put(prefix + EXCLUSIVE_LOCK, exclusiveLock.getPath());
+            data.put(prefix + EXCLUSIVE_LOCK, exclusiveLock.toString());
 
         if (nativeDir != null)
-            data.put(prefix + NATIVEDIR, nativeDir.getPath());
+            data.put(prefix + NATIVEDIR, nativeDir.toString());
 
         if (timeoutHandlerClassName != null)
             data.put(prefix + TIMEOUT_HANDLER, timeoutHandlerClassName);
@@ -773,7 +799,7 @@ public class RegressionParameters
         if (timeoutHandlerPath != null) {
             StringBuilder sb = new StringBuilder();
             String sep = "";
-            for (File file: timeoutHandlerPath) {
+            for (Path file: timeoutHandlerPath) {
                 sb.append(sep).append(file);
                 sep = File.pathSeparator;
             }
@@ -783,6 +809,18 @@ public class RegressionParameters
         if (timeoutHandlerTimeout != -1) {  // -1: default; 0: no timeout; >0: timeout in seconds
             data.put(prefix + TIMEOUT_HANDLER_TIMEOUT, String.valueOf(timeoutHandlerTimeout));
         }
+
+        if (testThreadFactory != null) {
+            data.put(prefix + CUSTOM_TEST_THREAD_FACTORY, testThreadFactory);
+        }
+
+        if (testThreadFactoryPath != null) {
+            data.put(prefix + CUSTOM_TEST_THREAD_FACTORY_PATH, testThreadFactoryPath);
+        }
+
+        if (testQueries != null) {
+            data.put(prefix + TEST_QUERIES, join(testQueries, "\n"));
+        }
     }
 
     //---------------------------------------------------------------------
@@ -863,8 +901,7 @@ public class RegressionParameters
     //---------------------------------------------------------------------
 
     public void setIgnoreKind(IgnoreKind ignoreKind) {
-        ignoreKind.getClass(); // null-check
-        this.ignoreKind = ignoreKind;
+        this.ignoreKind = Objects.requireNonNull(ignoreKind);
     }
 
     public IgnoreKind getIgnoreKind() {
@@ -888,8 +925,7 @@ public class RegressionParameters
     //---------------------------------------------------------------------
 
     public void setCompileJDK(JDK compileJDK) {
-        compileJDK.getClass(); // null check
-        this.compileJDK = compileJDK;
+        this.compileJDK = Objects.requireNonNull(compileJDK);
     }
 
     public JDK getCompileJDK() {
@@ -901,8 +937,7 @@ public class RegressionParameters
     //---------------------------------------------------------------------
 
     public void setTestJDK(JDK testJDK) {
-        testJDK.getClass(); // null check
-        this.testJDK = testJDK;
+        this.testJDK = Objects.requireNonNull(testJDK);
     }
 
     public JDK getTestJDK() {
@@ -914,8 +949,7 @@ public class RegressionParameters
     //---------------------------------------------------------------------
 
     public void setJUnitPath(SearchPath junitPath) {
-        junitPath.getClass(); // null check
-        this.junitPath = junitPath;
+        this.junitPath = Objects.requireNonNull(junitPath);
     }
 
     public SearchPath getJUnitPath() {
@@ -931,8 +965,7 @@ public class RegressionParameters
     //---------------------------------------------------------------------
 
     public void setTestNGPath(SearchPath testngPath) {
-        testngPath.getClass(); // null check
-        this.testngPath = testngPath;
+        this.testngPath = Objects.requireNonNull(testngPath);
     }
 
     public SearchPath getTestNGPath() {
@@ -948,8 +981,7 @@ public class RegressionParameters
     //---------------------------------------------------------------------
 
     public void setAsmToolsPath(SearchPath asmToolsPath) {
-        asmToolsPath.getClass(); // null check
-        this.asmToolsPath = asmToolsPath;
+        this.asmToolsPath = Objects.requireNonNull(asmToolsPath);
     }
 
     public SearchPath getAsmToolsPath() {
@@ -962,13 +994,22 @@ public class RegressionParameters
 
     public SearchPath getJavaTestClassPath() {
         if (javaTestClassPath == null) {
-            File jtClsDir = ProductInfo.getJavaTestClassDir();
+            Path jtClsDir = ProductInfo.getJavaTestClassDir().toPath();
             javaTestClassPath = new SearchPath(jtClsDir);
 
-            if (jtClsDir.getName().equals("javatest.jar")) {
-                File installDir = jtClsDir.getParentFile();
-                // append jtreg.jar to the path
-                javaTestClassPath.append(new File(installDir, "jtreg.jar"));
+            if (jtClsDir.getFileName().toString().equals("javatest.jar")) {
+                Path installDir = jtClsDir.getParent();
+                // append jtreg.jar or exploded directory to the search path
+                Path jtreg = installDir.resolve("jtreg.jar");
+                if (Files.exists(jtreg)) {
+                    javaTestClassPath.append(jtreg);
+                } else try {
+                    // use code source location of this class instead
+                    URL location = getClass().getProtectionDomain().getCodeSource().getLocation();
+                    javaTestClassPath.append(Path.of(location.toURI()));
+                } catch (Exception e) { // including NullPointerException and URISyntaxException
+                    throw new RuntimeException("Computation of Java test class-path failed", e);
+                }
             }
         }
         return javaTestClassPath;
@@ -1014,7 +1055,7 @@ public class RegressionParameters
         opts.addAll(getTestVMOptions());
         opts.addAll(getTestJavaOptions());
         if (nativeDir != null)
-            opts.add("-Djava.library.path=" + nativeDir.getAbsolutePath());
+            opts.add("-Djava.library.path=" + nativeDir.toAbsolutePath());
 
         return Collections.unmodifiableList(opts);
     }
@@ -1139,47 +1180,44 @@ public class RegressionParameters
 
     //---------------------------------------------------------------------
 
-    public void setReportDir(File reportDir) {
-        reportDir.getClass(); // null check
-        this.reportDir = reportDir;
+    public void setReportDir(Path reportDir) {
+        this.reportDir = Objects.requireNonNull(reportDir);
     }
 
-    public File getReportDir() {
+    public Path getReportDir() {
         return reportDir;
     }
 
-    private File reportDir;
+    private Path reportDir;
 
     //---------------------------------------------------------------------
 
-    public void setExclusiveLock(File exclusiveLock) {
-        exclusiveLock.getClass(); // null check
-        this.exclusiveLock = exclusiveLock;
+    public void setExclusiveLock(Path exclusiveLock) {
+        this.exclusiveLock = Objects.requireNonNull(exclusiveLock);
     }
 
-    public File getExclusiveLock() {
+    public Path getExclusiveLock() {
         return exclusiveLock;
     }
 
-    private File exclusiveLock;
+    private Path exclusiveLock;
 
     //---------------------------------------------------------------------
 
-    public void setNativeDir(File nativeDir) {
+    public void setNativeDir(Path nativeDir) {
         this.nativeDir = nativeDir;
     }
 
-    public File getNativeDir() {
+    public Path getNativeDir() {
         return nativeDir;
     }
 
-    private File nativeDir;
+    private Path nativeDir;
 
     //---------------------------------------------------------------------
 
     public void setTimeoutHandler(String timeoutHandlerClassName) {
-        timeoutHandlerClassName.getClass(); // null check
-        this.timeoutHandlerClassName = timeoutHandlerClassName;
+        this.timeoutHandlerClassName = Objects.requireNonNull(timeoutHandlerClassName);
     }
 
     String getTimeoutHandler() {
@@ -1191,25 +1229,24 @@ public class RegressionParameters
     //---------------------------------------------------------------------
 
     void setTimeoutHandlerPath(String timeoutHandlerPath) {
-        timeoutHandlerPath.getClass(); // null check
+        Objects.requireNonNull(timeoutHandlerPath);
         this.timeoutHandlerPath = new ArrayList<>();
         for (String f: timeoutHandlerPath.split(File.pathSeparator)) {
             if (f.length() > 0) {
-                this.timeoutHandlerPath.add(new File(f));
+                this.timeoutHandlerPath.add(Path.of(f));
             }
         }
     }
 
-    public void setTimeoutHandlerPath(List<File> timeoutHandlerPath) {
-        timeoutHandlerPath.getClass(); // null check
-        this.timeoutHandlerPath = timeoutHandlerPath;
+    public void setTimeoutHandlerPath(List<Path> timeoutHandlerPath) {
+        this.timeoutHandlerPath = Objects.requireNonNull(timeoutHandlerPath);
     }
 
-    List<File> getTimeoutHandlerPath() {
+    List<Path> getTimeoutHandlerPath() {
         return timeoutHandlerPath;
     }
 
-    private List<File> timeoutHandlerPath;
+    private List<Path> timeoutHandlerPath;
 
     //---------------------------------------------------------------------
 
@@ -1226,18 +1263,38 @@ public class RegressionParameters
     }
 
     private long timeoutHandlerTimeout;
-
     //---------------------------------------------------------------------
 
-    public void setMatchLists(File[] files) {
-        this.matchLists = Arrays.asList(files);
+    public void setTestThreadFactory(String testThreadFactory) {
+        this.testThreadFactory = testThreadFactory;
     }
 
-    List<File> getMatchLists() {
+    public String getTestThreadFactory() {
+        return testThreadFactory;
+    }
+
+    private String testThreadFactory;
+
+    public void setTestThreadFactoryPath(String testThreadFactoryPath) {
+        this.testThreadFactoryPath = testThreadFactoryPath;
+    }
+
+    public String getTestThreadFactoryPath() {
+        return testThreadFactoryPath;
+    }
+
+    private String testThreadFactoryPath;
+    //---------------------------------------------------------------------
+
+    public void setMatchLists(Path[] files) {
+        this.matchLists = List.of(files);
+    }
+
+    List<Path> getMatchLists() {
         return Collections.unmodifiableList(matchLists);
     }
 
-    private List<File> matchLists;
+    private List<Path> matchLists;
 
     //---------------------------------------------------------------------
 
@@ -1253,20 +1310,55 @@ public class RegressionParameters
 
     //---------------------------------------------------------------------
 
+    public void setTestQueries(List<String> testQueries) {
+        this.testQueries = testQueries;
+    }
+
+    public List<String> getTestQueries() {
+        return testQueries;
+    }
+
+    /**
+     * Returns the query component for a given test, if one was specified,
+     * or null if there is no query component for this test.
+     *
+     * @param test the name of the test
+     * @return the query component associated with this test, or null
+     */
+    public String getTestQuery(String test) {
+        // There are two common cases:
+        // 1. any number of tests are being run, none of which have queries, or
+        // 2. a single test is being run, which has query.
+        // As such, it is probably not worth parsing testQueries into a map.
+        if (testQueries != null) {
+            for (String tq : testQueries) {
+                int sep = tq.indexOf("?");
+                if (test.equals(tq.substring(0, sep))) {
+                    return tq.substring(sep + 1);
+                }
+            }
+        }
+        return null;
+    }
+
+    private List<String> testQueries;
+
+    //---------------------------------------------------------------------
+
     public Pattern getRefIgnoreLinesPattern() {
         if (refIgnoreLinesPattern == UNSET_PATTERN) {
             String refIgnoreLines = System.getenv("JTREG_REF_IGNORE_LINES");
             String re;
             if (refIgnoreLines != null) {
                 // User-specified list of regular expressions for lines to ignore in golden file comparison.
-                re = Arrays.asList(refIgnoreLines.trim().split("\\s+")).stream()
+                re = Arrays.stream(refIgnoreLines.trim().split("\\s+"))
                         .map(s -> "(" + s + ")")
                         .collect(Collectors.joining("|"));
             } else {
                 // Default regular expressions, based on VM warnings when specific powerful VM options are set.
                 // Override these by setting JTREG_REF_IGNORE_LINES to either empty or alternative regex list
                 Map<String, String> envVars = getEnvVars();
-                re = Arrays.asList("JAVA_TOOL_OPTIONS", "_JAVA_OPTIONS").stream()
+                re = Stream.of("JAVA_TOOL_OPTIONS", "_JAVA_OPTIONS")
                         .filter(envVars::containsKey)
                         .map(e -> "(Picked up " + e + ":.*)")
                         .collect(Collectors.joining("|"));
@@ -1282,7 +1374,7 @@ public class RegressionParameters
 
     }
 
-    private static Pattern UNSET_PATTERN = Pattern.compile("");
+    private static final Pattern UNSET_PATTERN = Pattern.compile("");
     private Pattern refIgnoreLinesPattern = UNSET_PATTERN;
 
     //---------------------------------------------------------------------
@@ -1321,7 +1413,7 @@ public class RegressionParameters
             put(map, "test.jdk", getTestJDK(), JDK::getAbsolutePath);
             put(map, "compile.jdk", getCompileJDK(), JDK::getAbsolutePath);
             put(map, "test.timeout.factor", getTimeoutFactor(), String::valueOf);
-            put(map, "test.nativepath", getNativeDir(), File::getAbsolutePath);
+            put(map, "test.nativepath", getNativeDir(), p -> p.toAbsolutePath().toString());
             put(map, "test.root", getTestSuite().getRootDir(), File::getAbsolutePath);
             basicTestProperties = map;
         }
@@ -1338,7 +1430,7 @@ public class RegressionParameters
 
     //---------------------------------------------------------------------
 
-    private OS getTestOS() {
+    public OS getTestOS() {
         // In general, and particularly when running tests, the testJDK should always be set.
         // But in some testing and reporting situations, it may not be. In these cases,
         // we default to the current platform.
@@ -1362,4 +1454,6 @@ public class RegressionParameters
     private List<String> retainArgs;
     private final Set<Integer> retainStatusSet = new HashSet<>(4);
     private Pattern retainFilesPattern;
+
+    private static final I18NResourceBundle i18n = I18NResourceBundle.getBundleForClass(RegressionParameters.class);
 }
