@@ -1,5 +1,5 @@
 /*
- * Copyright (c) 1997, 2019, Oracle and/or its affiliates. All rights reserved.
+ * Copyright (c) 1997, 2022, Oracle and/or its affiliates. All rights reserved.
  * DO NOT ALTER OR REMOVE COPYRIGHT NOTICES OR THIS FILE HEADER.
  *
  * This code is free software; you can redistribute it and/or modify it
@@ -32,11 +32,11 @@ import java.io.FileReader;
 import java.io.IOException;
 import java.io.StreamTokenizer;
 import java.lang.reflect.Field;
-import java.util.Arrays;
 import java.util.Collection;
 import java.util.HashMap;
 import java.util.HashSet;
 import java.util.LinkedHashSet;
+import java.util.List;
 import java.util.Map;
 import java.util.Set;
 import java.util.regex.Matcher;
@@ -50,6 +50,7 @@ import com.sun.javatest.finder.HTMLCommentStream;
 import com.sun.javatest.finder.JavaCommentStream;
 import com.sun.javatest.finder.ShScriptCommentStream;
 import com.sun.javatest.finder.TagTestFinder;
+import com.sun.javatest.regtest.agent.Flags;
 import com.sun.javatest.regtest.exec.Action;
 import com.sun.javatest.regtest.util.StringUtils;
 import com.sun.javatest.util.I18NResourceBundle;
@@ -57,12 +58,11 @@ import com.sun.javatest.util.I18NResourceBundle;
 /**
   * This is a specific implementation of the TagTestFinder which is to be used
   * for JDK regression testing.  It follows the test-tag specifications as given
-  * in http://openjdk.java.net/jtreg/tag-spec.txt.
+  * in http://openjdk.org/jtreg/tag-spec.txt.
   *
   * A test description consists of a single block comment in either Java files
   * or shell-script files.  A file may contain multiple test descriptions.
   *
-  * @author Iris A Garcia
   * @see com.sun.javatest.TestFinder
   * @see com.sun.javatest.finder.TagTestFinder
   */
@@ -95,15 +95,15 @@ public class RegressionTestFinder extends TagTestFinder
 
     @SuppressWarnings("unchecked")
     Set<String> getAllowedExtensions() {
-        return ((HashMap) getField("extensionTable")).keySet();
+        return ((Map<String, ?>) getField("extensionTable")).keySet();
     }
 
     @SuppressWarnings("unchecked")
     Set<String> getIgnoredDirectories() {
-        return ((HashMap) getField("excludeList")).keySet();
+        return ((Map<String, ?>) getField("excludeList")).keySet();
     }
 
-    Object getField(String name) {
+    private Object getField(String name) {
         try {
             Field f = TagTestFinder.class.getDeclaredField(name);
             try {
@@ -112,16 +112,10 @@ public class RegressionTestFinder extends TagTestFinder
             } finally {
                 f.setAccessible(false);
             }
-        } catch (NoSuchFieldException e) {
-            e.printStackTrace(System.err);
-            return null;
-        } catch (SecurityException e) {
-            e.printStackTrace(System.err);
-            return null;
-        } catch (IllegalArgumentException e) {
-            e.printStackTrace(System.err);
-            return null;
-        } catch (IllegalAccessException e) {
+        } catch (NoSuchFieldException
+                 | SecurityException
+                 | IllegalArgumentException
+                 | IllegalAccessException e) {
             e.printStackTrace(System.err);
             return null;
         }
@@ -134,17 +128,22 @@ public class RegressionTestFinder extends TagTestFinder
 
     @Override
     protected void scanFile(File file) {
-        // filter out SCCS leftovers
-        if (file.getName().startsWith(","))
-            return;
+        if (traceFinder) {
+            System.err.println("RegressionTestFinder: reading " + file);
+        }
 
         try {
             File tngRoot = properties.getTestNGRoot(file);
             if (tngRoot != null) {
                 scanTestNGFile(tngRoot, file);
             } else {
-                //super.scanFile(file);
-                modifiedScanFile(file);
+                File junitRoot = properties.getJUnitRoot(file);
+                if (junitRoot != null) {
+                    scanJUnitFile(junitRoot, file);
+                } else {
+                    //super.scanFile(file);
+                    modifiedScanFile(file);
+                }
             }
         } catch (TestSuite.Fault e) {
             error(i18n, "finder.cant.read.test.properties", e.getMessage());
@@ -184,8 +183,7 @@ public class RegressionTestFinder extends TagTestFinder
         if (dot == -1)
             return;
         String extn = name.substring(dot);
-        @SuppressWarnings("unchecked")
-        Class<? extends CommentStream> csc = (Class<? extends CommentStream>) getClassForExtension(extn);
+        Class<? extends CommentStream> csc = getClassForExtension(extn);
         if (csc == null) {
             error(super_i18n, "tag.noParser", file, extn);
             return;
@@ -208,8 +206,7 @@ public class RegressionTestFinder extends TagTestFinder
             String comment = cs.readComment();
             int commentLine = r.lineNumber;
             while (comment != null) {
-                @SuppressWarnings({"unchecked", "cast"}) // temporary, to cover transition generifying TestFinder
-                Map<String,String> tagValues = (Map<String,String>) parseComment(comment, file);
+                Map<String,String> tagValues = parseComment(comment, file);
 
                 // Look ahead to see if there are more comments
                 String nextComment = cs.readComment();
@@ -260,7 +257,7 @@ public class RegressionTestFinder extends TagTestFinder
         }
     }
 
-    private class LineCounterBufferedReader extends BufferedReader {
+    private static class LineCounterBufferedReader extends BufferedReader {
         int lineNumber;
 
         LineCounterBufferedReader(FileReader r) {
@@ -298,48 +295,59 @@ public class RegressionTestFinder extends TagTestFinder
     }
 
     protected void scanTestNGFile(File tngRoot, File file) throws TestSuite.Fault {
-        Map<String,String> tagValues;
         if (isTestNGTest(file)) {
-            PackageImportParser p = new PackageImportParser(tngRoot, file);
-            p.parse();
-            String className = p.inferClassName();
-            try (BufferedReader in = new BufferedReader(new FileReader(file))) {
-                tagValues = readTestNGComments(file, in);
-                if (tagValues == null) {
-                    tagValues = new HashMap<>();
-                    // could read more of file looking for annotations like @Test, @Factory
-                    // to guess whether this is really a test file or not
-                }
+            scanFile(tngRoot, file, "testngClass", true);
+        }
+    }
 
-                tagValues.put("packageRoot", getRootDir().toURI().relativize(tngRoot.toURI()).getPath());
-                if (className == null) {
-                    tagValues.put("error", "cannot determine class name");
-                } else {
-                    tagValues.put("testngClass", className);
-                }
-                if (p.importsJUnit)
-                    tagValues.put("importsJUnit", "true");
+    protected void scanJUnitFile(File junitRoot, File file) throws TestSuite.Fault {
+        if (isJUnitTest(file)) {
+            scanFile(junitRoot, file, "junitClass", false);
+        }
+    }
 
-                Set<String> libDirs = properties.getLibDirs(file);
-                if (libDirs != null && !libDirs.isEmpty()) {
-                    tagValues.put("library", StringUtils.join(libDirs, " "));
-                }
-
-                foundTestDescription(tagValues, file, /*line*/0);
-            } catch (IOException e) {
-                error(i18n, "finder.ioError", file);
+    protected void scanFile(File junitRoot, File file, String classPropertyName, boolean setImportsJUnit) throws TestSuite.Fault {
+        Map<String,String> tagValues;
+        PackageImportParser p = new PackageImportParser(junitRoot, file);
+        p.parse();
+        String className = p.inferClassName();
+        try (BufferedReader in = new BufferedReader(new FileReader(file))) {
+            tagValues = readComments(file, in);
+            if (tagValues == null) {
+                tagValues = new HashMap<>();
+                // could read more of file looking for annotations like @Test, @Factory
+                // to guess whether this is really a test file or not
             }
+
+            tagValues.put("packageRoot", getRootDir().toURI().relativize(junitRoot.toURI()).getPath());
+            if (className == null) {
+                tagValues.put("error", "cannot determine class name");
+            } else {
+                tagValues.put(classPropertyName, className);
+            }
+            if (setImportsJUnit &&  p.importsJUnit) {
+                tagValues.put("importsJUnit", "true");
+            }
+
+            Set<String> libDirs = properties.getLibDirs(file);
+            if (libDirs != null && !libDirs.isEmpty()) {
+                tagValues.put("library", StringUtils.join(libDirs, " "));
+            }
+
+            foundTestDescription(tagValues, file, /*line*/0);
+        } catch (IOException e) {
+            error(i18n, "finder.ioError", file);
         }
     }
 
     private class PackageImportParser {
-        private final File tngRoot;
+        private final File rootDir;
         private final File file;
         String packageName;
         boolean importsJUnit;
 
-        PackageImportParser(File tngRoot, File file) {
-            this.tngRoot = tngRoot;
+        PackageImportParser(File rootDir, File file) {
+            this.rootDir = rootDir;
             this.file = file;
         }
 
@@ -395,7 +403,7 @@ public class RegressionTestFinder extends TagTestFinder
         }
 
         String inferClassName() {
-            String path = tngRoot.toURI().relativize(file.toURI()).getPath();
+            String path = rootDir.toURI().relativize(file.toURI()).getPath();
             String fn = file.getName();
             String cn = fn.replace(".java", "");
             String pkg_fn = (packageName == null) ? file.getName() : packageName.replace('.', '/') + "/" + fn;
@@ -410,7 +418,7 @@ public class RegressionTestFinder extends TagTestFinder
         }
     }
 
-    private Map<String,String> readTestNGComments(File file, BufferedReader in) throws IOException {
+    private Map<String,String> readComments(File file, BufferedReader in) throws IOException {
         CommentStream cs = new JavaCommentStream();
         cs.init(in);
         cs.setFastScan(true);
@@ -419,7 +427,6 @@ public class RegressionTestFinder extends TagTestFinder
         String comment;
         int index = 1;
         while ((comment = cs.readComment()) != null) {
-            @SuppressWarnings("unchecked")
             Map<String, String> tv = parseComment(comment, file);
             if (tv.isEmpty())
                 continue;
@@ -446,6 +453,16 @@ public class RegressionTestFinder extends TagTestFinder
     protected boolean isTestNGTest(File file) {
         // for now, ignore comments and annotations, and
         // assume *.java is a test
+        return isClassOrInterfaceFile(file);
+    }
+
+    protected boolean isJUnitTest(File file) {
+        // for now, ignore comments and annotations, and
+        // assume *.java is a test
+        return isClassOrInterfaceFile(file);
+    }
+
+    private boolean isClassOrInterfaceFile(File file) {
         String name = file.getName();
         return name.endsWith(".java")
                 && !name.equals("module-info.java")
@@ -460,28 +477,31 @@ public class RegressionTestFinder extends TagTestFinder
         }
     }
 
-    @Override @SuppressWarnings({"unchecked", "rawtypes"})
-    protected Map<String, String> normalize(Map tv) {
-        return normalize0((Map<String, String>) tv);
-    }
-
-    private Map<String, String> normalize0(Map<String, String> tagValues) {
+    protected Map<String, String> normalize(Map<String, String> tagValues) {
         Map<String, String> newTagValues = new HashMap<>();
         String fileName = getCurrentFile().getName();
         String baseName = fileName.substring(0, fileName.lastIndexOf("."));
-        boolean testNG = tagValues.containsKey("testngClass");
+        boolean isTestNG = tagValues.containsKey("testngClass");
+        boolean isJUnit = tagValues.containsKey("junitClass");
 
         // default values
         newTagValues.put("title", " ");
         newTagValues.put("source", fileName);
 
-        if (testNG) {
+        if (isTestNG) {
             if (tagValues.get("run") != null) {
                 tagValues.put("error", PARSE_BAD_RUN);
             }
             String className = tagValues.get("testngClass");
             newTagValues.put("run", Action.REASON_ASSUMED_ACTION + " testng "
                              + className + LINESEP);
+        } else if (isJUnit) {
+            if (tagValues.get("run") != null) {
+                tagValues.put("error", PARSE_BAD_RUN);
+            }
+            String className = tagValues.get("junitClass");
+            newTagValues.put("run", Action.REASON_ASSUMED_ACTION + " junit "
+                    + className + LINESEP);
         } else if (fileName.endsWith(".sh")) {
             newTagValues.put("run", Action.REASON_ASSUMED_ACTION + " shell "
                              + fileName + LINESEP);
@@ -571,10 +591,10 @@ public class RegressionTestFinder extends TagTestFinder
         if (match(value, SHELL_ACTION))
             keywords.add("shell");
 
-        if (match(value, JUNIT_ACTION))
+        if (match(value, JUNIT_ACTION) || isJUnit)
             keywords.add("junit");
 
-        if (match(value, TESTNG_ACTION) || testNG)
+        if (match(value, TESTNG_ACTION) || isTestNG)
             keywords.add("testng");
 
         if (match(value, DRIVER_ACTION))
@@ -650,29 +670,21 @@ public class RegressionTestFinder extends TagTestFinder
     private Set<String> split(String s, String regex) {
         Set<String> result = new LinkedHashSet<>();
         if (s != null) {
-            result.addAll(Arrays.asList(s.split(regex)));
+            result.addAll(List.of(s.split(regex)));
         }
         return result;
     }
 
     /**
-     * Make sure that the provide name-value pair is of the proper format as
+     * Make sure that the provided name-value pair is of the proper format as
      * described in the tag-spec.
      *
      * @param entries   The map of the entries being read
      * @param name      The name of the entry that has been read
      * @param value     The value of the entry that has been read
      */
-    @Override @SuppressWarnings({"unchecked", "rawtypes"})
-    protected void processEntry(Map entries, String name, String value)
+    protected void processEntry(Map<String, String> entries, String name, String value)
     {
-        Map<String, String> tagValues = (Map<String, String>) entries;
-
-        // check for valid tag name, don't produce error message for the
-        // the SCCS sequence '%' 'W' '%'
-        if (name.startsWith("(#)"))
-            return;
-
         // translate the shorthands into run actions
         if (name.startsWith(COMPILE)
             || name.startsWith(CLEAN)
@@ -685,27 +697,27 @@ public class RegressionTestFinder extends TagTestFinder
         try {
             switch (name) {
                 case RUN:
-                    processRun(tagValues, value);
+                    processRun(entries, value);
                     break;
 
                 case BUG:
-                    processBug(tagValues, value);
+                    processBug(entries, value);
                     break;
 
                 case REQUIRES:
-                    processRequires(tagValues, value);
+                    processRequires(entries, value);
                     break;
 
                 case KEY:
-                    processKey(tagValues, value);
+                    processKey(entries, value);
                     break;
 
                 case MODULES:
-                    processModules(tagValues, value);
+                    processModules(entries, value);
                     break;
 
                 case LIBRARY:
-                    processLibrary(tagValues, value);
+                    processLibrary(entries, value);
                     break;
 
                 case COMMENT:
@@ -713,18 +725,18 @@ public class RegressionTestFinder extends TagTestFinder
                     break;
 
                 case ENABLE_PREVIEW:
-                    processEnablePreview(tagValues, value);
+                    processEnablePreview(entries, value);
                     break;
 
                 default:
                     if (!validTagNames.contains(name)) {
-                        parseError(tagValues, PARSE_TAG_BAD + name);
+                        parseError(entries, PARSE_TAG_BAD + name);
                     } else {
-                        tagValues.put(name, value);
+                        entries.put(name, value);
                     }
             }
         } catch (TestSuite.Fault e) {
-            reportError(tagValues, e.getMessage());
+            reportError(entries, e.getMessage());
         }
     }
 
@@ -756,8 +768,7 @@ public class RegressionTestFinder extends TagTestFinder
 
     private void reportError(Map<String, String> tagValues, String value) {
         // for now, just record first error
-        if (tagValues.get(ERROR) == null)
-            tagValues.put(ERROR, value);
+        tagValues.putIfAbsent(ERROR, value);
     }
 
     /**
@@ -837,12 +848,7 @@ public class RegressionTestFinder extends TagTestFinder
             return;
         }
 
-        String oldValue = tagValues.get(REQUIRES);
-        if (oldValue == null) {
-            tagValues.put(REQUIRES, value);
-        } else {
-            tagValues.put(REQUIRES, "(" + oldValue + ") & (" + value + ")");
-        }
+        tagValues.merge(REQUIRES, value, (a, b) -> "(" + a + ") & (" + b + ")");
     }
 
     /**
@@ -852,7 +858,6 @@ public class RegressionTestFinder extends TagTestFinder
      *
      * @param tagValues The map of all of the current tag values.
      * @param value     The value of the entry currently being processed.
-     * @return    A string which contains the new value for the "key" tag.
      */
     private void processKey(Map<String, String> tagValues, String value)
             throws TestSuite.Fault {
@@ -883,7 +888,6 @@ public class RegressionTestFinder extends TagTestFinder
      * Analyse the contents of @modules.
      * @param tagValues The map of all of the current tag values.
      * @param value     The value of the entry currently being processed.
-     * @return    A string which contains the new value for the "key" tag.
      */
     private void processModules(Map<String, String> tagValues, String value)
             throws TestSuite.Fault {
@@ -892,7 +896,7 @@ public class RegressionTestFinder extends TagTestFinder
             return;
         }
 
-        processModules(tagValues, Arrays.asList(value.trim().split("\\s+")));
+        processModules(tagValues, List.of(value.trim().split("\\s+")));
     }
 
     private void processModules(Map<String, String> tagValues, Collection<String> modules) {
@@ -914,12 +918,11 @@ public class RegressionTestFinder extends TagTestFinder
     }
 
     /**
-     * Create the library-directory list.  Pathnames are prepended left to
+     * Create the library-directory list.  Path names are prepended left to
      * right.
      *
      * @param tagValues The map of all of the current tag values.
      * @param value     The value of the entry currently being processed.
-     * @return    A string which contains the new value for the "library" tag.
      */
     private void processLibrary(Map<String, String> tagValues, String value) {
         String newValue;
@@ -1008,13 +1011,9 @@ public class RegressionTestFinder extends TagTestFinder
 
     private static final String LINESEP = System.getProperty("line.separator");
 
-    static final String[] excludeNames = {
-        "SCCS", "Codemgr_wsdata", ".hg", "RCS", ".svn",
-        "DeletedFiles", "DELETED-FILES", "deleted_files",
-        "TemporarilyRemoved"
-    };
+    static final String[] excludeNames = { ".hg", ".git" };
 
-    // These are all of the error messages using in the finder.
+    // These are all the error messages used in the finder.
     protected static final String
         PARSE_TAG_BAD         = "Invalid tag: ",
         PARSE_BUG_EMPTY       = "No value provided for `@bug'",
@@ -1064,4 +1063,6 @@ public class RegressionTestFinder extends TagTestFinder
     private static final I18NResourceBundle i18n = I18NResourceBundle.getBundleForClass(RegressionTestFinder.class);
     private static final boolean rejectTrailingBuild =
             !Boolean.getBoolean("javatest.regtest.allowTrailingBuild");
+
+    private static final boolean traceFinder = Flags.get("traceFinder");
 }
