@@ -1,5 +1,5 @@
 /*
- * Copyright (c) 2010, 2020, Oracle and/or its affiliates. All rights reserved.
+ * Copyright (c) 2010, 2023, Oracle and/or its affiliates. All rights reserved.
  * DO NOT ALTER OR REMOVE COPYRIGHT NOTICES OR THIS FILE HEADER.
  *
  * This code is free software; you can redistribute it and/or modify it
@@ -53,7 +53,6 @@ import java.util.LinkedList;
 import java.util.List;
 import java.util.Map;
 import java.util.Objects;
-import java.util.Queue;
 import java.util.Set;
 import java.util.TreeMap;
 import java.util.TreeSet;
@@ -95,7 +94,8 @@ public class Agent {
      * Start a JDK with given JVM options.
      */
     private Agent(File dir, JDK jdk, List<String> vmOpts, Map<String, String> envVars,
-            File policyFile, float timeoutFactor, Logger logger) throws Fault {
+            File policyFile, float timeoutFactor, Logger logger,
+            String testThreadFactory, String testThreadFactoryPath) throws Fault {
         try {
             id = ++count;
             this.jdk = jdk;
@@ -104,10 +104,13 @@ public class Agent {
             this.logger = logger;
 
             List<String> cmd = new ArrayList<>();
-            cmd.add(jdk.getJavaProg().getPath());
+            cmd.add(jdk.getJavaProg().toString());
             cmd.addAll(vmOpts);
             if (policyFile != null)
                 cmd.add("-Djava.security.policy=" + policyFile.toURI());
+            String headless = System.getProperty("java.awt.headless");
+            if (headless != null)
+                cmd.add("-Djava.awt.headless=" + headless);
             cmd.add(AgentServer.class.getName());
 
             cmd.add(AgentServer.ID);
@@ -132,6 +135,15 @@ public class Agent {
                 cmd.add(String.valueOf(timeoutFactor));
             }
 
+            if (testThreadFactory != null) {
+                cmd.add(AgentServer.CUSTOM_TEST_THREAD_FACTORY);
+                cmd.add(testThreadFactory);
+            }
+
+            if (testThreadFactoryPath != null) {
+                cmd.add(CUSTOM_TEST_THREAD_FACTORY_PATH);
+                cmd.add(testThreadFactoryPath);
+            }
             log("Started " + cmd);
 
             ProcessBuilder pb = new ProcessBuilder(cmd);
@@ -736,7 +748,9 @@ public class Agent {
         synchronized Agent getAgent(File dir,
                                     JDK jdk,
                                     List<String> vmOpts,
-                                    Map<String, String> envVars)
+                                    Map<String, String> envVars,
+                                    String testThreadFactory,
+                                    String testThreadFactoryPath)
                 throws Fault {
             logger.log(null,
                     "POOL: get agent for:\n"
@@ -754,7 +768,8 @@ public class Agent {
                 stats.reuse(a);
             } else {
                 logger.log(null, "POOL: Creating new agent");
-                a = new Agent(dir, jdk, vmOpts, envVars, policyFile, timeoutFactor, logger);
+                a = new Agent(dir, jdk, vmOpts, envVars, policyFile, timeoutFactor, logger,
+                        testThreadFactory, testThreadFactoryPath);
                 stats.add(a);
             }
 
@@ -887,7 +902,7 @@ public class Agent {
         }
 
         private static String getKey(File dir, JDK jdk, List<String> vmOpts) {
-            return (dir.getAbsolutePath() + " " + jdk.getAbsoluteFile() + " " + StringUtils.join(vmOpts, " "));
+            return (dir.getAbsolutePath() + " " + jdk.getAbsoluteHomeDirectory() + " " + StringUtils.join(vmOpts, " "));
         }
 
         private final Logger logger;
