@@ -1,5 +1,5 @@
 /*
- * Copyright (c) 1998, 2020, Oracle and/or its affiliates. All rights reserved.
+ * Copyright (c) 1998, 2023, Oracle and/or its affiliates. All rights reserved.
  * DO NOT ALTER OR REMOVE COPYRIGHT NOTICES OR THIS FILE HEADER.
  *
  * This code is free software; you can redistribute it and/or modify it
@@ -30,8 +30,8 @@ import java.io.File;
 import java.io.FileWriter;
 import java.io.IOException;
 import java.io.PrintWriter;
+import java.nio.file.Path;
 import java.util.ArrayList;
-import java.util.Arrays;
 import java.util.Collections;
 import java.util.EnumSet;
 import java.util.LinkedHashMap;
@@ -66,7 +66,6 @@ import static com.sun.javatest.regtest.RStatus.passed;
  * This class implements the "main" action as described by the JDK tag
  * specification.
  *
- * @author Iris A Garcia
  * @see Action
  * @see com.sun.javatest.regtest.agent.MainActionHelper
  */
@@ -158,7 +157,7 @@ public class MainAction extends Action
                     String name = optValue;
                     if (optValue.startsWith("=")) {
                         overrideSysPolicy = true;
-                        name = optValue.substring(1, optValue.length());
+                        name = optValue.substring(1);
                     }   policyFN = parsePolicy(name);
                     break;
                 case "secure":
@@ -181,7 +180,7 @@ public class MainAction extends Action
 
         if (driverClass != null) {
             this.driverClass = driverClass;
-            this.driverArgs = Arrays.asList(driverArgs);
+            this.driverArgs = List.of(driverArgs);
         }
 
         if (script.useBootClassPath()) {
@@ -284,7 +283,7 @@ public class MainAction extends Action
         Set<File> files = new LinkedHashSet<>();
         if (testClassName != null) {
             Map<String,String> buildOpts = Collections.emptyMap();
-            List<String> buildArgs = Arrays.asList(join(testModuleName, testClassName));
+            List<String> buildArgs = List.of(join(testModuleName, testClassName));
             try {
                 BuildAction ba = new BuildAction();
                 ba.init(buildOpts, buildArgs, SREASON_ASSUMED_BUILD, script);
@@ -300,7 +299,7 @@ public class MainAction extends Action
     @Override
     public Set<String> getModules() {
         return (testModuleName == null)
-                ? Collections.<String>emptySet()
+                ? Collections.emptySet()
                 : Collections.singleton(testModuleName);
     }
 
@@ -374,7 +373,7 @@ public class MainAction extends Action
         // though an "@run build <class>" action had been inserted before
         // this action."
         Map<String,String> buildOpts = Collections.emptyMap();
-        List<String> buildArgs = Arrays.asList(join(testModuleName, testClassName));
+        List<String> buildArgs = List.of(join(testModuleName, testClassName));
         BuildAction ba = new BuildAction();
         return ba.build(buildOpts, buildArgs, SREASON_ASSUMED_BUILD, script);
     }
@@ -429,7 +428,7 @@ public class MainAction extends Action
             env.put("CLASSPATH", cp.toString());
         }
 
-        String javaCmd = script.getJavaProg();
+        Path javaCmd = script.getJavaProg();
         JDKOpts javaOpts = new JDKOpts();
 
         if (!useCLASSPATH) {
@@ -455,6 +454,13 @@ public class MainAction extends Action
             }
         }
 
+        if (driverClass != null && testModuleName != null && testClassName.contains(".")) {
+            Module driverModule = driverClass.getModule();
+            String exportTarget = driverModule.isNamed() ? driverModule.getName() : "ALL-UNNAMED";
+            String testPackage = testClassName.substring(0, testClassName.lastIndexOf("."));
+            javaOpts.add("--add-exports=" + testModuleName + "/" + testPackage + "=" + exportTarget);
+        }
+
         javaOpts.addAll(getExtraModuleConfigOptions(Modules.Phase.DYNAMIC));
         javaOpts.addAll(script.getTestVMJavaOptions());
         javaOpts.addAll(script.getTestDebugOptions());
@@ -474,6 +480,13 @@ public class MainAction extends Action
         else if (policyFN != null) {
             javaProps.put("java.security.manager", "default");
         }
+
+        if (script.getTestThreadFactory() != null) {
+            javaProps.put(MainWrapper.TEST_THREAD_FACTORY, script.getTestThreadFactory());
+        }
+        if (script.getTestThreadFactoryPath() != null) {
+            javaProps.put(MainWrapper.TEST_THREAD_FACTORY_PATH, script.getTestThreadFactoryPath());
+        }
 //      javaProps.put("java.security.debug", "all");
 
         javaOpts.addAll(testJavaArgs);
@@ -485,7 +498,7 @@ public class MainAction extends Action
         classArgs.addAll(runClassArgs);
 
         List<String> command = new ArrayList<>();
-        command.add(javaCmd);
+        command.add(javaCmd.toString());
         for (Map.Entry<String,String> e: javaProps.entrySet())
             command.add("-D" + e.getKey() + "=" + e.getValue());
         command.addAll(filterJavaOpts(javaOpts.toList()));
@@ -494,9 +507,9 @@ public class MainAction extends Action
 
         // PASS TO PROCESSCOMMAND
         Status status;
-        PrintWriter sysOut = section.createOutput("System.out");
-        PrintWriter sysErr = section.createOutput("System.err");
-        try {
+        try (PrintWriter sysOut = section.createOutput("System.out");
+             PrintWriter sysErr = section.createOutput("System.err")) {
+
             if (showMode)
                 showMode(getName(), ExecMode.OTHERVM, section);
             if (showCmd)
@@ -506,7 +519,7 @@ public class MainAction extends Action
 
             // RUN THE MAIN WRAPPER CLASS
             ProcessCommand cmd = new ProcessCommand();
-            cmd.setExecDir(script.absTestScratchDir());
+            cmd.setExecDir(script.absTestScratchDir().toFile());
 
             // Set the exit codes and their associated strings.  Note that we
             // require the use of a non-zero exit code for a passed test so
@@ -517,19 +530,16 @@ public class MainAction extends Action
             cmd.setDefaultStatus(failed(UNEXPECT_SYS_EXIT));
 
             TimeoutHandler timeoutHandler =
-                script.getTimeoutHandlerProvider().createHandler(this.getClass(), script, section);
+                    script.getTimeoutHandlerProvider().createHandler(this.getClass(), script, section);
 
             cmd.setCommand(command)
-                .setEnvironment(env)
-                .setStreams(sysOut, sysErr)
-                .setTimeout(timeout, TimeUnit.SECONDS)
-                .setTimeoutHandler(timeoutHandler);
+                    .setEnvironment(env)
+                    .setStreams(sysOut, sysErr)
+                    .setTimeout(timeout, TimeUnit.SECONDS)
+                    .setTimeoutHandler(timeoutHandler);
 
             status = normalize(cmd.exec());
 
-        } finally {
-            sysOut.close();
-            sysErr.close();
         }
 
         // EVALUATE THE RESULTS
@@ -570,7 +580,7 @@ public class MainAction extends Action
                 script.getExecutionPaths(multiModule, runModuleName, useBootClassPath, true);
 
         JDK jdk = script.getTestJDK();
-        List<File> stdLibs = new SearchPath()
+        List<Path> stdLibs = new SearchPath()
                 .append(script.getJavaTestClassPath())
                 .append(jdk.getJDKClassPath())
                 .append(script.getJUnitPath())
@@ -600,7 +610,7 @@ public class MainAction extends Action
         // "test.src" and "test.classes", respectively"
         Map<String, String> javaProps = script.getTestProperties();
 
-        String javaProg = script.getJavaProg();
+        Path javaProg = script.getJavaProg();
         List<String> javaArgs = new ArrayList<>();
         javaArgs.add("-classpath");
         javaArgs.add(classpath.toString());
@@ -617,8 +627,11 @@ public class MainAction extends Action
 
         Agent agent;
         try {
+            String factory = script.getTestThreadFactory() == null ? null : script.getTestThreadFactory();
             agent = script.getAgent(jdk, agentClasspath,
-                    filterJavaOpts(join(script.getTestVMJavaOptions(), script.getTestDebugOptions())));
+                    filterJavaOpts(join(script.getTestVMJavaOptions(), script.getTestDebugOptions())),
+                    factory,
+                    script.getTestThreadFactoryPath());
             section.getMessageWriter().println("Agent id: " + agent.getId());
             new ModuleConfig("Boot Layer").setFromOpts(agent.vmOpts).write(configWriter);
         } catch (Agent.Fault e) {
@@ -787,5 +800,5 @@ public class MainAction extends Action
     protected Set<String> othervmOverrideReasons = new LinkedHashSet<>();
     protected boolean nativeCode = false;
     private int     timeout = -1;
-    private String  manual  = "unset";
+    private String  manual  = "unset"; // or "novalue"
 }
