@@ -1,5 +1,5 @@
 /*
- * Copyright (c) 1997, 2020, Oracle and/or its affiliates. All rights reserved.
+ * Copyright (c) 1997, 2023, Oracle and/or its affiliates. All rights reserved.
  * DO NOT ALTER OR REMOVE COPYRIGHT NOTICES OR THIS FILE HEADER.
  *
  * This code is free software; you can redistribute it and/or modify it
@@ -25,20 +25,18 @@
 
 package com.sun.javatest.regtest.exec;
 
-import java.io.BufferedInputStream;
-import java.io.BufferedOutputStream;
 import java.io.File;
-import java.io.FileInputStream;
-import java.io.FileOutputStream;
-import java.io.IOException;
-import java.io.InputStream;
-import java.io.OutputStream;
 import java.io.PrintWriter;
+import java.io.UncheckedIOException;
 import java.lang.reflect.Method;
 import java.net.InetAddress;
 import java.net.MalformedURLException;
 import java.net.UnknownHostException;
+import java.nio.file.Files;
+import java.nio.file.InvalidPathException;
+import java.nio.file.Path;
 import java.util.ArrayList;
+import java.util.Arrays;
 import java.util.Collections;
 import java.util.EnumMap;
 import java.util.HashMap;
@@ -61,6 +59,7 @@ import com.sun.javatest.TestEnvironment;
 import com.sun.javatest.TestResult;
 import com.sun.javatest.TestSuite;
 import com.sun.javatest.regtest.agent.JDK_Version;
+import com.sun.javatest.regtest.agent.MainWrapper;
 import com.sun.javatest.regtest.agent.SearchPath;
 import com.sun.javatest.regtest.config.ExecMode;
 import com.sun.javatest.regtest.config.Expr;
@@ -70,12 +69,14 @@ import com.sun.javatest.regtest.config.JDKOpts;
 import com.sun.javatest.regtest.config.Locations;
 import com.sun.javatest.regtest.config.Locations.LibLocn;
 import com.sun.javatest.regtest.config.Modules;
+import com.sun.javatest.regtest.config.OS;
 import com.sun.javatest.regtest.config.ParseException;
 import com.sun.javatest.regtest.config.RegressionEnvironment;
 import com.sun.javatest.regtest.config.RegressionParameters;
 import com.sun.javatest.regtest.config.RegressionTestSuite;
-import com.sun.javatest.regtest.report.TestNGReporter;
+import com.sun.javatest.regtest.report.SummaryReporter;
 import com.sun.javatest.regtest.tool.Version;
+import com.sun.javatest.regtest.util.FileUtils;
 import com.sun.javatest.regtest.util.StringUtils;
 
 import static com.sun.javatest.regtest.RStatus.error;
@@ -85,7 +86,6 @@ import static com.sun.javatest.regtest.RStatus.passed;
   * This class interprets the TestDescription as specified by the JDK tag
   * specification.
   *
-  * @author Iris A Garcia
   * @see com.sun.javatest.Script
   */
 public class RegressionScript extends Script {
@@ -141,6 +141,11 @@ public class RegressionScript extends Script {
         }
         testResult.putProperty("jtregVersion", getVersion());
         testResult.putProperty("testJDK", getTestJDK().getAbsolutePath());
+        OS testOs = params.getTestOS();
+        testResult.putProperty("testJDK_OS", testOs.toString());
+        testResult.putProperty("testJDK_os.name", testOs.name);
+        testResult.putProperty("testJDK_os.version", testOs.version);
+        testResult.putProperty("testJDK_os.arch", testOs.arch);
         if (!getCompileJDK().equals(getTestJDK())) {
             testResult.putProperty("compileJDK", getCompileJDK().getAbsolutePath());
         }
@@ -180,15 +185,14 @@ public class RegressionScript extends Script {
             needJUnit = false;
             needTestNG = false;
 
-            if (td.getParameter("importsJUnit") != null) {
-                needJUnit = true;
-                needTestNG = true;
-            } else {
-                for (Action a: actionList) {
-                    if (a instanceof JUnitAction) {
+            for (Action a : actionList) {
+                if (a instanceof JUnitAction) {
+                    needJUnit = true;
+                } else if (a instanceof TestNGAction) {
+                    needTestNG = true;
+                    // check for using mixed-mode
+                    if (td.getParameter("importsJUnit") != null) {
                         needJUnit = true;
-                    } else if (a instanceof TestNGAction) {
-                        needTestNG = true;
                     }
                 }
             }
@@ -205,8 +209,8 @@ public class RegressionScript extends Script {
             } else {
                 // check actions for test-specific modules
                 actionLoop:
-                for (Action a: actionList) {
-                    for (String m: a.getModules()) {
+                for (Action a : actionList) {
+                    for (String m : a.getModules()) {
                         if (systemModules.contains(m)) {
                             usePatchModules = true;
                             break actionLoop;
@@ -231,11 +235,11 @@ public class RegressionScript extends Script {
                     // output for default case unchanged
                     printJDKInfo(msgPW, "JDK under test", getTestJDK(), getTestVMOptions());
                 } else {
-                    printJDKInfo(msgPW, "compile JDK", getCompileJDK(), Collections.<String>emptyList());
+                    printJDKInfo(msgPW, "compile JDK", getCompileJDK(), Collections.emptyList());
                     printJDKInfo(msgPW, "test JDK", getTestJDK(), getTestVMOptions());
                 }
 
-                for (LibLocn lib: locations.getLibs()) {
+                for (LibLocn lib : locations.getLibs()) {
                     String kind;
                     switch (lib.kind) {
                         case PACKAGE:
@@ -257,14 +261,14 @@ public class RegressionScript extends Script {
                     if (lib.absSrcDir != null) {
                         msgPW.println("   source directory: " + lib.absSrcDir);
                     }
-                    if (lib.absClsDir.isFile() && lib.absClsDir.getName().endsWith(".jar")) {
+                    if (Files.isRegularFile(lib.absClsDir) && lib.absClsDir.getFileName().toString().endsWith(".jar")) {
                         msgPW.println("   jar file: " + lib.absClsDir);
                     } else {
                         msgPW.println("   class directory: " + lib.absClsDir);
                     }
                 }
 
-                while (! actionList.isEmpty()) {
+                while (!actionList.isEmpty()) {
                     Action action = actionList.remove();
                     status = action.run();
                     if (status.getType() != Status.PASSED)
@@ -283,8 +287,19 @@ public class RegressionScript extends Script {
                 | Modules.Fault
                 | ParseActionsException
                 | TestRunException
-                | TestSuite.Fault e) {
+                | TestSuite.Fault
+                | FileUtils.NIOFileOperationException e) {
             status = error(e.getMessage());
+        } catch (UncheckedIOException e) {
+            String msg = "IO exception: " + e.getMessage();
+            if (e.getCause() != null)
+                msg += " (" + e.getCause() + ")";
+            status = error(msg);
+        } catch (InvalidPathException e) {
+            String msg = "Invalid path: " + e.getInput() + "; " + e.getMessage();
+            if (e.getCause() != null)
+                msg += " (" + e.getCause() + ")";
+            status = error(msg);
         } finally {
             int elapsed = (int) (System.currentTimeMillis() - started);
             int millis = (elapsed % 1000);
@@ -322,7 +337,7 @@ public class RegressionScript extends Script {
     private void printJDKInfo(PrintWriter pw, String label, JDK jdk, List<String> opts) {
         pw.print(label);
         pw.print(": ");
-        pw.println(jdk.getAbsoluteFile());
+        pw.println(jdk.getAbsoluteHomeDirectory());
         String v = jdk.getVersionText(opts, pw::println);
         if (v.length() > 0) {
             pw.println(v);
@@ -404,9 +419,7 @@ public class RegressionScript extends Script {
                 opts.put(keyValue[0], keyValue[1]);
                 // [[fail,], [ref, Foo.ref]]
             }
-            List<String> args = new ArrayList<>();
-            for (int i = 2; i < tokens.length; i++)
-                args.add(tokens[i]);
+            List<String> args = new ArrayList<>(Arrays.asList(tokens).subList(2, tokens.length));
             // [-debug, Foo.java] (everything after the big options token)
             Class<?> c = null;
             try {
@@ -444,7 +457,7 @@ public class RegressionScript extends Script {
 
         boolean fast = true;
         for (String arg : args) {
-            fast = fast && (arg.indexOf("${") == -1);
+            fast = fast && (!arg.contains("${"));
         }
         if (fast) {
             return args;
@@ -456,12 +469,12 @@ public class RegressionScript extends Script {
         return newArgs;
     }
 
-    private static final Pattern namePattern = Pattern.compile("\\$\\{([A-Za-z0-9._]+)\\}");
+    private static final Pattern namePattern = Pattern.compile("\\$\\{([A-Za-z0-9._]+)}");
 
     private static String evalNames(String arg, Expr.Context c, Map<String,String> testProps)
             throws Expr.Fault, ParseException {
         Matcher m = namePattern.matcher(arg);
-        StringBuffer sb = null;
+        StringBuilder sb = null;
         // Note that '\' may appear in the replacement value for paths on Windows,
         // and so, in the following loop, avoid using Matcher::appendReplacement,
         // which interprets '\' and `$' in the replacement string.
@@ -469,14 +482,14 @@ public class RegressionScript extends Script {
         int pos = 0;
         while (m.find(pos)) {
             if (sb == null) {
-                sb = new StringBuffer();
+                sb = new StringBuilder();
             }
             String name = m.group(1);
             String value = testProps.containsKey(name) ? testProps.get(name) : c.get(name);
             if ("null".equals(value)) {
                 throw new ParseException("unset property " + name);
             }
-            sb.append(arg.substring(pos, m.start()));
+            sb.append(arg, pos, m.start());
             sb.append(value);
             pos = m.end();
         }
@@ -492,7 +505,7 @@ public class RegressionScript extends Script {
 
     /**
      * Get the timeout to be used for a test.  Since the timeout for regression
-     * tests is on a per action basis rather than on a per test basis, this
+     * tests is on a per-action basis rather than on a per-test basis, this
      * method should always return zero which indicates that there is no
      * timeout.
      *
@@ -504,22 +517,22 @@ public class RegressionScript extends Script {
     }
 
     /**
-     * Get the timeout to be used for an action.  The timeout will be scaled by
-     * the timeoutFactor as necessary.  The default timeout for any action as
-     * per the tag-spec is 120 seconds scaled by a value found in the
-     * environment ("javatestTimeoutFactor").
-     * The timeout factor is available as both an integer (for backward
-     * compatibility) and a floating point number
+     * Returns the timeout to be used for an action.
      *
-     * @param time The initial timeout which may need to be scaled according
-     *             to the provided timeoutFactor.  If the initial timeout is
-     *             less than zero, then the default timeout will be returned.
-     * @return     The timeout in seconds.
+     * If no debug options have been set, the result will be the given value,
+     * or a default value if the given value is negative, scaled by the timeout factor.
+     *
+     * If debug options have been set, the result will be 0, meaning "no timeout".
+     *
+     * @param time the value of the timeout specified in the action,
+     *             or -1 if no value was specified
+     * @return     the timeout, in seconds
      */
     protected int getActionTimeout(int time) {
-        if (time < 0)
-            time = 120;
-        return (int) (time * getTimeoutFactor());
+        final int DEFAULT_ACTION_TIMEOUT = 120; // seconds
+        return isTimeoutsEnabled()
+                ? (int) ((time < 0 ? DEFAULT_ACTION_TIMEOUT : time) * getTimeoutFactor())
+                : 0;
     }
 
     protected float getTimeoutFactor() {
@@ -527,12 +540,14 @@ public class RegressionScript extends Script {
             // not synchronized, so in worst case may be set more than once
             float value = 1; // default
             try {
-                // use [1] to get the floating point timeout factor
+                // The timeout factor is available as both an integer (for backward compatibility)
+                // and a floating point number.
+                // Use [1] to get the floating point timeout factor
                 String f = (regEnv == null ? null : regEnv.lookup("javatestTimeoutFactor")[1]);
                 if (f != null)
                     value = Float.parseFloat(f);
-            } catch (TestEnvironment.Fault e) {
-            } catch (NumberFormatException e) {
+            } catch (TestEnvironment.Fault
+                     | NumberFormatException e) {
             }
             cacheJavaTestTimeoutFactor = value;
         }
@@ -541,6 +556,16 @@ public class RegressionScript extends Script {
 
     private static float cacheJavaTestTimeoutFactor = -1;
 
+    /**
+     * Returns whether timeouts are (generally) enabled.
+     *
+     * @return {@code true} if timeouts are enabled, and {@code false} otherwise
+     */
+    protected boolean isTimeoutsEnabled() {
+        // for now, timeouts are always enabled, unless debug options have been specified for the test
+        return getTestDebugOptions().isEmpty();
+    }
+
     /**
      * Set an alarm that will interrupt the calling thread after a specified
      * delay (in milliseconds), and repeatedly thereafter until canceled.  The
@@ -581,7 +606,7 @@ public class RegressionScript extends Script {
      */
     private String getReason(String[] cmd) {
         String retVal;
-        StringBuffer sb = new StringBuffer();
+        StringBuilder sb = new StringBuilder();
         String reason = cmd[0];
         switch (reason) {
             case Action.REASON_ASSUMED_ACTION:
@@ -672,7 +697,7 @@ public class RegressionScript extends Script {
     /**
      * Path to native components.
      */
-    File getNativeDir() {
+    Path getNativeDir() {
         return params.getNativeDir();
     }
 
@@ -707,31 +732,36 @@ public class RegressionScript extends Script {
         }
     }
 
+    String getTestQuery() {
+        String testName = testResult.getTestName();
+        return params.getTestQuery(testName);
+    }
+
     //----------------------- computing paths ---------------------------------
 
-    File absTestWorkFile(String name) {
+    Path absTestWorkFile(String name) {
         return locations.absTestWorkFile(name);
     }
 
-    File absTestSrcDir() {
+    Path absTestSrcDir() {
         return locations.absTestSrcDir();
     } // absTestSrcDir()
 
-    File absTestClsDir() {
+    Path absTestClsDir() {
         return locations.absTestClsDir();
     } // absTestClsDir()
 
-    File absTestScratchDir() {
-        return scratchDirectory.dir.getAbsoluteFile();
+    Path absTestScratchDir() {
+        return scratchDirectory.dir.toPath().toAbsolutePath();
     } // absTestScratchDir()
 
-    File absTestClsTopDir() {
+    Path absTestClsTopDir() {
         return locations.absBaseClsDir();
     } // absTestClsTopDir()
 
-    private boolean useBootClassPath(File classdir) throws TestClassException {
+    private boolean useBootClassPath(Path classdir) throws TestClassException {
         try {
-            String rel = locations.absBaseClsDir().toURI().relativize(classdir.toURI()).getPath();
+            String rel = locations.absBaseClsDir().toUri().relativize(classdir.toFile().toURI()).getPath();
             return testSuite.useBootClassPath(rel);
         } catch (TestSuite.Fault f) {
             throw new TestClassException(f.toString());
@@ -761,7 +791,7 @@ public class RegressionScript extends Script {
             if (multiModule) {
                 msp.append(locations.absTestSrcDir());
             } else {
-                File testSrcDir = locations.absTestSrcDir(module);
+                Path testSrcDir = locations.absTestSrcDir(module);
                 sp.append(testSrcDir);
                 // Ideally, the source directory need only go on the source path
                 // but some tests rely on precompiled .class files existing in
@@ -789,7 +819,7 @@ public class RegressionScript extends Script {
             if (multiModule)
                 msp.append(libLocn.absSrcDir);
             else if (module != null)
-                sp.append(new File(libLocn.absSrcDir, module));
+                sp.append(libLocn.absSrcDir.resolve(module));
         }
 
         if (module == null) {
@@ -814,12 +844,8 @@ public class RegressionScript extends Script {
                 // Put necessary jar files onto the module path as automatic modules.
                 // We cannot use the ${jtreg.home}/lib directory directly since it contains
                 // other jar files which are not valid as automatic modules.
-                File md = workDir.getFile("modules");
-                if (needJUnit)
-                    install(params.getJUnitPath(), md);
-                if (needTestNG)
-                    install(params.getTestNGPath(), md);
-                mp.append(md);
+                if (needJUnit) params.getJUnitPath().asList().forEach(mp::append);
+                if (needTestNG) params.getTestNGPath().asList().forEach(mp::append);
             }
         } else {
             if (needJUnit)
@@ -915,12 +941,8 @@ public class RegressionScript extends Script {
                 // Put necessary jar files onto the module path as automatic modules.
                 // We cannot use the ${jtreg.home}/lib directory directly since it contains
                 // other jar files which are not valid as automatic modules.
-                File md = workDir.getFile("modules");
-                if (needJUnit)
-                    install(params.getJUnitPath(), md);
-                if (needTestNG)
-                    install(params.getTestNGPath(), md);
-                mp.append(md);
+                if (needJUnit) params.getJUnitPath().asList().forEach(mp::append);
+                if (needTestNG) params.getTestNGPath().asList().forEach(mp::append);
             }
         } else {
             SearchPath fp = (!bcp.isEmpty() || usePatchModules()) ? bcp : cp;
@@ -964,39 +986,6 @@ public class RegressionScript extends Script {
         return map;
     }
 
-    private void install(SearchPath path, File dir) throws TestRunException {
-        synchronized (params.getWorkDirectory()) { // avoid multiple tests doing simultaneous install
-            if (dir.exists()) {
-                if (!dir.isDirectory())
-                    throw new TestRunException("modules in work directory is not a directory");
-            } else {
-                dir.mkdirs();
-            }
-
-            for (File jar: path.asList()) {
-                File target = new File(dir, jar.getName());
-                if (target.exists()
-                        && target.length() == jar.length()
-                        && target.lastModified() == jar.lastModified()) {
-                    return;
-                }
-
-                // Eventually replace with java.nio.file.Files::copy
-                try (OutputStream out = new BufferedOutputStream(new FileOutputStream(target));
-                        InputStream in = new BufferedInputStream(new FileInputStream(jar))) {
-                    byte[] buf = new byte[8192];
-                    int n;
-                    while ((n = in.read(buf)) != -1) {
-                        out.write(buf, 0, n);
-                    }
-                } catch (IOException e) {
-                    throw new TestRunException("cannot init modules directory", e);
-                }
-                target.setLastModified(jar.lastModified());
-            }
-        }
-    }
-
     boolean useBootClassPath() {
         return useBootClassPath;
     }
@@ -1006,10 +995,10 @@ public class RegressionScript extends Script {
     }
 
     boolean hasTestPatchMods() {
-        File testModulesDir = locations.absTestPatchDir();
-        if (testModulesDir.isDirectory()) {
-            for (File f: testModulesDir.listFiles()) {
-                if (f.isDirectory())
+        Path testModulesDir = locations.absTestPatchDir();
+        if (Files.isDirectory(testModulesDir)) {
+            for (Path f : FileUtils.listFiles(testModulesDir)) {
+                if (Files.isDirectory(f))
                     return true;
             }
         }
@@ -1022,10 +1011,10 @@ public class RegressionScript extends Script {
     }
 
     boolean hasTestUserMods() {
-        File testModulesDir = locations.absTestModulesDir();
-        if (testModulesDir.isDirectory()) {
-            for (File f: testModulesDir.listFiles()) {
-                if (f.isDirectory())
+        Path testModulesDir = locations.absTestModulesDir();
+        if (Files.isDirectory(testModulesDir)) {
+            for (Path f : FileUtils.listFiles(testModulesDir)) {
+                if (Files.isDirectory(f))
                     return true;
             }
         }
@@ -1060,8 +1049,12 @@ public class RegressionScript extends Script {
         return params.getAsmToolsPath();
     }
 
-    TestNGReporter getTestNGReporter() {
-        return TestNGReporter.instance(workDir);
+    SummaryReporter getTestNGSummaryReporter() {
+        return SummaryReporter.forTestNG(workDir);
+    }
+
+    SummaryReporter getJUnitSummaryReporter() {
+        return SummaryReporter.forJUnit(workDir);
     }
 
     Lock getLockIfRequired() throws TestRunException {
@@ -1088,6 +1081,7 @@ public class RegressionScript extends Script {
     //--------------------------------------------------------------------------
 
     TimeoutHandlerProvider getTimeoutHandlerProvider() throws TestRunException {
+
         try {
             return params.getTimeoutHandlerProvider();
         } catch (MalformedURLException e) {
@@ -1095,6 +1089,14 @@ public class RegressionScript extends Script {
         }
     }
 
+    String getTestThreadFactory() {
+        return params.getTestThreadFactory();
+    }
+
+    String getTestThreadFactoryPath() {
+        return params.getTestThreadFactoryPath();
+    }
+
     //--------------------------------------------------------------------------
 
     JDK getTestJDK() {
@@ -1105,8 +1107,8 @@ public class RegressionScript extends Script {
         return getTestJDK().getVersion(params, msgPW::println);
     }
 
-    String getJavaProg() {
-        return params.getTestJDK().getJavaProg().getPath();
+    Path getJavaProg() {
+        return params.getTestJDK().getJavaProg();
     }
 
     //--------------------------------------------------------------------------
@@ -1119,8 +1121,8 @@ public class RegressionScript extends Script {
         return getCompileJDK().getVersion(params, msgPW::println);
     }
 
-    String getJavacProg() {
-        return params.getCompileJDK().getJavacProg().getPath();
+    Path getJavacProg() {
+        return params.getCompileJDK().getJavacProg();
     }
 
     //--------------------------------------------------------------------------
@@ -1131,11 +1133,16 @@ public class RegressionScript extends Script {
         // initialize the properties with standard properties common to all tests
         Map<String, String> p = new LinkedHashMap<>(params.getBasicTestProperties());
         // add test-specific properties
-        p.put("test.name", testResult.getTestName());
-        p.put("test.file", locations.absTestFile().getPath());
-        p.put("test.src", locations.absTestSrcDir().getPath());
+        String testName = testResult.getTestName();
+        p.put("test.name", testName);
+        String testQuery = params.getTestQuery(testName);
+        if (testQuery != null) {
+            p.put("test.query", testQuery);
+        }
+        p.put("test.file", locations.absTestFile().toString());
+        p.put("test.src", locations.absTestSrcDir().toString());
         p.put("test.src.path", toString(locations.absTestSrcPath()));
-        p.put("test.classes", locations.absTestClsDir().getPath());
+        p.put("test.classes", locations.absTestClsDir().toString());
         p.put("test.class.path", toString(locations.absTestClsPath()));
         if (getExecMode() == ExecMode.AGENTVM) {
             // The following will be added to javac.class.path on the test VM
@@ -1165,11 +1172,17 @@ public class RegressionScript extends Script {
         return Collections.unmodifiableMap(p);
     }
     // where
-    private String toString(List<File> files) {
+    private String toString(List<Path> files) {
         return files.stream()
-                .map(File::getPath)
+                .map(Path::toString)
                 .collect(Collectors.joining(File.pathSeparator));
     }
+//    // where
+//    private String toString(List<File> files) {
+//        return files.stream()
+//                .map(File::getPath)
+//                .collect(Collectors.joining(File.pathSeparator));
+//    }
 
     File getTestRootDir() {
         return params.getTestSuite().getRootDir();
@@ -1180,12 +1193,19 @@ public class RegressionScript extends Script {
     /*
      * Get an agent for a VM with the given VM options.
      */
-    Agent getAgent(JDK jdk, SearchPath classpath, List<String> testVMOpts) throws Agent.Fault {
+    Agent getAgent(JDK jdk, SearchPath classpath, List<String> testVMOpts,
+                   String testThreadFactory, String testThreadFactoryPath) throws Agent.Fault {
         JDKOpts vmOpts = new JDKOpts();
         vmOpts.addAll("-classpath", classpath.toString());
         vmOpts.addAll(testVMOpts);
+
+        if (testThreadFactory != null) {
+            // Add property to differ agents with and without MainWrapper
+            vmOpts.add("-D" + MainWrapper.TEST_THREAD_FACTORY + "=" + testThreadFactory);
+        }
+
         if (params.getTestJDK().hasModules()) {
-            vmOpts.addAllPatchModules(new SearchPath(params.getWorkDirectory().getFile("patches")));
+            vmOpts.addAllPatchModules(new SearchPath(params.getWorkDirectory().getFile("patches").toPath()));
         }
 
         /*
@@ -1195,13 +1215,12 @@ public class RegressionScript extends Script {
          * record the agents that the script has already obtained for use.
          */
         for (Agent agent: agents) {
-            if (agent.matches(absTestScratchDir(), jdk, vmOpts.toList())) {
+            if (agent.matches(absTestScratchDir().toFile(), jdk, vmOpts.toList())) {
                 return agent;
             }
         }
 
-        Map<String, String> envVars = new HashMap<>();
-        envVars.putAll(getEnvVars());
+        Map<String, String> envVars = new HashMap<>(getEnvVars());
         // some tests are inappropriately relying on the CLASSPATH environment
         // variable being set, so ensure it is set. See equivalent code in MainAction
         // and Main.execChild. Note we cannot set exactly the same classpath as
@@ -1210,7 +1229,8 @@ public class RegressionScript extends Script {
         envVars.put("CLASSPATH", cp.toString());
 
         Agent.Pool p = Agent.Pool.instance(params);
-        Agent agent = p.getAgent(absTestScratchDir(), jdk, vmOpts.toList(), envVars);
+        Agent agent = p.getAgent(absTestScratchDir().toFile(), jdk, vmOpts.toList(), envVars,
+                testThreadFactory, testThreadFactoryPath);
         agents.add(agent);
         return agent;
     }
@@ -1257,8 +1277,8 @@ public class RegressionScript extends Script {
 
     //----------internal classes-----------------------------------------------
 
-    void saveScratchFile(File file, File dest) {
-        scratchDirectory.retainFile(file, dest);
+    void saveScratchFile(Path file, Path dest) {
+        scratchDirectory.retainFile(file.toFile(), dest.toFile());
     }
 
     //----------internal classes-----------------------------------------------
