@@ -1,5 +1,5 @@
 /*
- * Copyright (c) 1998, 2019, Oracle and/or its affiliates. All rights reserved.
+ * Copyright (c) 1998, 2022, Oracle and/or its affiliates. All rights reserved.
  * DO NOT ALTER OR REMOVE COPYRIGHT NOTICES OR THIS FILE HEADER.
  *
  * This code is free software; you can redistribute it and/or modify it
@@ -27,9 +27,9 @@ package com.sun.javatest.regtest.exec;
 
 import java.io.File;
 import java.io.PrintWriter;
+import java.nio.file.Files;
 import java.nio.file.Path;
 import java.util.ArrayList;
-import java.util.Collections;
 import java.util.LinkedHashMap;
 import java.util.List;
 import java.util.Map;
@@ -50,7 +50,6 @@ import static com.sun.javatest.regtest.RStatus.*;
  * This class implements the "shell" action as described by the JDK tag
  * specification.
  *
- * @author Iris A Garcia
  * @see Action
  */
 public class ShellAction extends Action
@@ -159,7 +158,7 @@ public class ShellAction extends Action
 
     @Override
     public Set<File> getSourceFiles() {
-        return Collections.singleton(new File(script.absTestSrcDir(), shellFN));
+        return Set.of(script.absTestSrcDir().resolve(shellFN).toFile());
     }
 
     /**
@@ -186,7 +185,7 @@ public class ShellAction extends Action
 
         startAction(false);
 
-        File shellFile = new File(script.absTestSrcDir(), shellFN);
+        File shellFile = script.absTestSrcDir().resolve(shellFN).toFile();
         if (!shellFile.exists())
             throw new TestRunException(CANT_FIND_SRC + shellFile);
 
@@ -196,7 +195,7 @@ public class ShellAction extends Action
         if (script.isCheck()) {
             status = passed(CHECK_PASS);
         } else {
-            mkdirs(script.absTestClsDir());
+            mkdirs(script.absTestClsDir().toFile());
 
             // CONSTRUCT THE COMMAND LINE
 
@@ -226,19 +225,23 @@ public class ShellAction extends Action
             Modules modules = script.getModules();
             if (!modules.isEmpty())
                 env.put("TESTMODULES", modules.toString());
-            File nativeDir = script.getNativeDir();
+            Path nativeDir = script.getNativeDir();
             if (nativeDir != null) {
-                env.put("TESTNATIVEPATH", nativeDir.getAbsolutePath());
+                env.put("TESTNATIVEPATH", nativeDir.toAbsolutePath().toString());
             }
             if (script.enablePreview()) {
                 env.put("TESTENABLEPREVIEW", "true");
             }
+            String testQuery = script.getTestQuery();
+            if (testQuery != null) {
+                env.put("TESTQUERY", testQuery);
+            }
 
             List<String> command = new ArrayList<>();
             if (script.useWindowsSubsystemForLinux()) {
-                File java_exe = new File(new File(script.getTestJDK().getFile(), "bin"), "java.exe");
+                Path java_exe = script.getTestJDK().getHomeDirectory().resolve("bin").resolve("java.exe");
                 env.put("NULL", "/dev/null");
-                if (java_exe.exists()) {
+                if (Files.exists(java_exe)) {
                     // invoking a Windows binary: use standard Windows separator characters
                     env.put("EXE_SUFFIX", ".exe");
                     env.put("FS", "/");
@@ -290,7 +293,7 @@ public class ShellAction extends Action
 
                 // RUN THE SHELL SCRIPT
                 ProcessCommand cmd = new ProcessCommand()
-                    .setExecDir(script.absTestScratchDir())
+                    .setExecDir(script.absTestScratchDir().toFile())
                     .setCommand(command)
                     .setEnvironment(env)
                     .setStreams(sysOut, sysErr)
@@ -334,17 +337,20 @@ public class ShellAction extends Action
         return status;
     } // run()
         // where
-        private String fixupSep(List<File> files) {
+        private String fixupSep(List<Path> files) {
             StringBuilder sb = new StringBuilder();
-            for (File f: files) {
+            for (Path f: files) {
                 if (sb.length() > 0) sb.append(File.pathSeparator);
                 sb.append(fixupSep(f));
             }
             return sb.toString();
         }
-        private String fixupSep(File f) {
-            return fixupSep(f.getPath());
-        }
+    private String fixupSep(Path f) {
+        return fixupSep(f.toString());
+    }
+    private String fixupSep(File f) {
+        return fixupSep(f.getPath());
+    }
         private String fixupSep(String s) {
             return (sep == null ? s : s.replace(File.separator, sep));
         }
@@ -416,5 +422,5 @@ public class ShellAction extends Action
 
     private boolean reverseStatus = false;
     private int     timeout = -1;
-    private String  manual  = "unset";
+    private String  manual  = "unset"; // or "novalue"
 }
